#include <algorithm>
#include <bitset>
#include <cassert>
#include <climits>
#include <cmath>
#include <iomanip>
#include <iostream>
#include <map>
#include <queue>
#include <set>
#include <tuple>
#include <unordered_map>
#include <unordered_set>
#include <vector>
using namespace std;

constexpr int N     = 100;
constexpr int LIM_V = 10, M = 2 * LIM_V + 1; // 9 + 8 + ... + 2 + 1 + 2 + ... + 8 + 9 = 89

typedef pair<int, int> ii;
#define x first
#define y second
#define pb push_back
#define eb emplace_back

bitset<N * N * M * M> seen;

int get_idx(int i, int j, int v, int w) {
	int idx = i * N + j;
	assert(0 <= v + LIM_V && v + LIM_V < M);
	assert(0 <= w + LIM_V && w + LIM_V < M);
	idx = idx * M + (v + LIM_V);
	return idx * M + (w + LIM_V);
}

bool cell[N][N];

int main() {
	string line;
	int rows, cols, k;
	cin >> rows >> cols >> k;
	k += 1;
	getline(cin, line);

	vector<ii> pos(k + 1);
	for(int i = 0; i < rows; i++) {
		getline(cin, line);
		for(int j = 0; j < cols; j++) {
			cell[i][j] = (line[j] != '#');
			if('0' <= line[j] && line[j] <= '9') {
				pos[line[j] - '0'] = ii(i, j);
				// cerr << (line[j] - '0') << " : " << i << ", " << j << endl;
			}
		}
	}

	vector<vector<int>> minTime(M, vector<int>(M, INT_MAX));
	minTime[LIM_V][LIM_V] = 0;

	for(int p = 1; p <= k; p++) {
		// calculate delta.
		seen.reset();

		ii fr = pos[p - 1], to = pos[p];

		vector<vector<int>> nextMinTime(M, vector<int>(M, INT_MAX));
		vector<pair<int, ii>> starts;
		for(int v = 0; v < M; v++)
			for(int w = 0; w < M; w++)
				if(minTime[v][w] != INT_MAX) starts.eb(minTime[v][w], ii(v - LIM_V, w - LIM_V));

		sort(starts.begin(), starts.end());
		if(starts.empty()) {
			puts("impossible");
			return 0;
		}

		/* cout << "p=" << p << " ";
		for (auto pr : starts) {
		    cout << pr.x << ": (" << pr.y.x << ", " << pr.y.y << ")" << endl;
		} */

		queue<int> q, nq;
		int cur_time = starts[0].x, cur_it = 0;

		while(true) {
			while(cur_it < (int)starts.size() && starts[cur_it].x == cur_time) {
				ii velocity = starts[cur_it++].y;
				q.push(get_idx(fr.x, fr.y, velocity.x, velocity.y));
			}

			if(cur_it == (int)starts.size() && q.empty()) {
				// we are done!
				break;
			}

			while(!q.empty()) {
				int cur = q.front();
				q.pop();

				int w = (cur % M) - LIM_V;
				cur /= M;
				int v = (cur % M) - LIM_V;
				cur /= M;
				int j = cur % N;
				cur /= N;
				int i = cur;

				// cerr << "CURRENT " << _t << " : " << i << " " << j << " " << v << " " << w <<
				// endl;

				if(ii(i, j) == to) {
					assert(nextMinTime[v + LIM_V][w + LIM_V] == INT_MAX);
					nextMinTime[v + LIM_V][w + LIM_V] = cur_time;
				}

				for(int dx = -1; dx <= 1; dx++) {
					for(int dy = -1; dy <= 1; dy++) {
						int nv = v + dx, nw = w + dy;
						if(nv >= LIM_V || nv <= -LIM_V || nw >= LIM_V || nw <= -LIM_V) continue;

						int ni = i + nv, nj = j + nw;
						if(ni < 0 || ni >= rows || nj < 0 || nj >= cols || !cell[i + nv][j + nw])
							continue;
						int nxt = get_idx(ni, nj, nv, nw);
						// cerr << "now at " << ni << ", " << nj << " (v=" << nv << ", " << nw <<
						// ")" << endl;
						if(!seen.test(nxt)) {
							seen.set(nxt, true);
							nq.push(nxt);
							// cerr << "enqueue " << nxt << " -> " << ni << " " << nj << " " << nv
							// << " " << nw << endl;
						}
					}
				}
			}

			cur_time++;
			assert(q.empty());
			swap(q, nq);
			// cerr << "next size: " << q.size() << " / " << nq.size() << endl;
		}

		minTime = nextMinTime;
	}

	int ans = minTime[0 + LIM_V][0 + LIM_V];
	if(ans == INT_MAX)
		puts("impossible");
	else
		cout << ans << endl;
	return 0;
}
