// @EXPECTED_RESULTS@: ACCEPTED, TIME_LIMIT_EXCEEDED
// This is an n^3 solution.
// Ideally TIME_LIMIT_EXCEEDED, but in practice most likely ACCEPTED because python and java n^2 is
// slower.
#include <algorithm>
#include <cassert>
#include <cmath>
#include <iomanip>
#include <iostream>
#include <map>
#include <set>
#include <tuple>
#include <unordered_map>
#include <unordered_set>
#include <vector>

using namespace std;

struct point {
	int x, y;
};

int main() {

	int n;
	cin >> n;

	std::vector<point> points(n);
	for(int i = 0; i < n; i++) { cin >> points[i].x >> points[i].y; }

	//    ___
	//   /   \      1 0
	//  |     |    2   7
	//   \___/     3   6
	//	            4 5

	// x ->
	// y ^

	std::vector<std::vector<std::vector<int>>> edges(8, std::vector<std::vector<int>>(n));

	// horizontal pieces
	// 1<-0
	// 4->5
	{
		std::map<int, std::vector<int>> slices;
		for(int i = 0; i < n; i++) { slices[points[i].y].push_back(i); }

		for(auto v : slices) {
			int slice = v.first;
			int elts  = v.second.size();
			if(elts > 1) {
				for(int i = 0; i < elts; i++) {
					for(int j = i + 1; j < elts; j++) {
						int a = v.second[i];
						int b = v.second[j];
						if(points[a].x > points[b].x) swap(a, b);

						// assume a < b (not double points)
						edges[0][b].push_back(a);
						edges[4][a].push_back(b);
					}
				}
			}
		}
	}

	// vertical pieces
	// 2 to 3
	// 6 to 7
	{
		std::map<int, std::vector<int>> slices;
		for(int i = 0; i < n; i++) { slices[points[i].x].push_back(i); }

		for(auto v : slices) {
			int elts = v.second.size();
			if(elts > 1) {
				for(int i = 0; i < elts; i++) {
					for(int j = i + 1; j < elts; j++) {
						int a = v.second[i];
						int b = v.second[j];
						if(points[a].y > points[b].y) swap(a, b);

						// assume a < b (not double points)
						edges[2][b].push_back(a);
						edges[6][a].push_back(b);
					}
				}
			}
		}
	}

	// \ pieces
	// 3 to 4
	// 7 to 8
	{
		std::map<int, std::vector<int>> slices;
		for(int i = 0; i < n; i++) { slices[points[i].x + points[i].y].push_back(i); }

		for(auto v : slices) {
			int elts = v.second.size();
			if(elts > 1) {
				for(int i = 0; i < elts; i++) {
					for(int j = i + 1; j < elts; j++) {
						int a = v.second[i];
						int b = v.second[j];
						if(points[a].x < points[b].x) swap(a, b);
						// assume a < b (not double points)
						edges[3][b].push_back(a);
						edges[7][a].push_back(b);
					}
				}
			}
		}
	}

	// / pieces
	// 1 to 2
	// 5 to 6
	{
		std::map<int, std::vector<int>> slices;
		for(int i = 0; i < n; i++) { slices[points[i].x - points[i].y].push_back(i); }

		for(auto v : slices) {
			int slice = v.first;
			int elts  = v.second.size();
			if(elts > 1) {
				for(int i = 0; i < elts; i++) {
					for(int j = i + 1; j < elts; j++) {
						int a = v.second[i];
						int b = v.second[j];
						if(points[a].x > points[b].x) swap(a, b);
						// assume a < b (not double points)
						edges[1][b].push_back(a);
						edges[5][a].push_back(b);
					}
				}
			}
		}
	}

	/*
	    for( int i = 0; i < 8; i++ ) {
	        std::cerr << "Ëdges " << i << std::endl;
	        for( int j = 0; j < n; j++ ) {
	            for( int k : edges[i][j] )
	                std::cout << j << " -> " << k << std::endl;
	        }

	    }
	*/
	// For each vertices i compute number of paths from (0,i) to (8,i)
	long long total_cnt = 0;
	for(int v = 0; v < n; v++) {
		std::vector<std::vector<long long>> cnts(9, std::vector<long long>(n, 0));
		cnts[0][v] = 1;
		for(int i = 0; i < 8; i++) {
			for(int j = 0; j < n; j++) {
				long long cnt = cnts[i][j];
				if(cnt == 0) continue;
				for(int k : edges[i][j]) { cnts[i + 1][k] += cnt; }
			}
		}
		total_cnt += cnts[8][v];
	}

	std::cout << total_cnt << std::endl;
}
